#include "ldrser.h"
#include "ldrbase.h"
#include "ldrblock.h"

STD_string LDRserJDX::get_prefix(const LDRbase& ldr) const {
  STD_string result;
  if(ldr.get_typeInfo()=="LDRblock") {
    result="##TITLE="+ldr.get_label()+"\n";
    result+="##JCAMPDX=4.24\n";
    result+="##DATATYPE=Parameter Values\n";
  } else {
    result="##";
    if(ldr.get_jdx_props().userdef_parameter) result+="$";
    result+=ldr.get_label();
    result+="=";
  }
  return result;
}

STD_string LDRserJDX::get_postfix(const LDRbase& ldr) const {
  if(ldr.get_typeInfo()=="LDRblock") {
    return "##END=\n";
  }
  return "\n";
}


STD_string LDRserJDX::extract_valstring(const STD_string& parstring) const {
  Log<LDRcomp> odinlog("LDRserJDX","extract_valstring");

  STD_string result;

  ODINLOG(odinlog,normalDebug) << "parstring: >" << parstring << "<" << STD_endl;

  STD_string from_hashes=extract(parstring,"##",""); // from first hash to the end
  ODINLOG(odinlog,normalDebug) << "from_hashes: >" << from_hashes << "<" << STD_endl;
  if(from_hashes=="") return result;

  STD_string to_hashes;
  if(noccur(from_hashes,"\n##")>0) {
    to_hashes=extract(from_hashes,"","\n##"); // to the next double hash
  } else {
    to_hashes=from_hashes;
  }
  ODINLOG(odinlog,normalDebug) << "to_hashes: >" << to_hashes << "<" << STD_endl;


  result=extract(to_hashes,"=",""); // everything after equal sign

  ODINLOG(odinlog,normalDebug) << "result: >" << result << "<" << STD_endl;

  return result;
}


STD_string LDRserJDX::remove_comments(const STD_string& parstring) const {
  STD_string result;

  result=rmblock(parstring, "\n$$", "\n", true, false); // Remove single-line comments
  result=rmblock(result, "$$", "\n", true, false); // Remove remaining end-line comments

  return result;
}


STD_string LDRserJDX::escape_characters(const STD_string& parstring) const {
/*
  Log<LDRcomp> odinlog("LDRserJDX","escape_characters");
  ODINLOG(odinlog,normalDebug) << "parstring: >" << parstring << "<" << STD_endl;

  STD_string result;
  if (parstring.length()>=2 && parstring[(unsigned int)0]=='<' && parstring[parstring.length()-1]=='>') {
    result="\""+parstring+"\"";
  } else {
    result=parstring;
  }
  ODINLOG(odinlog,normalDebug) << "result: >" << result << "<" << STD_endl;
  return result;
*/
  return parstring;
}

STD_string LDRserJDX::deescape_characters(const STD_string& parstring) const {
/*
  STD_string result;
  STD_string extracted=extract(parstring,"\"<",">\"",true);
  if(extracted!="") {
    result="<"+extracted+">";
  } else {
    result=parstring;
  }
  return result;
*/
  return parstring;
}


STD_string LDRserJDX::print_string(const STD_string& str) const {
  Log<LDRcomp> odinlog("LDRserJDX","print_string");
  STD_string result;
  if(get_jdx_compatmode()==bruker) {
    ndim nn(1);
    int l=str.length()*_BRUKER_MODE_STRING_CAP_FACTOR_;
    if(!l) l=_BRUKER_MODE_STRING_CAP_START_;
    if(l<_BRUKER_MODE_STRING_MIN_SIZE_) l=_BRUKER_MODE_STRING_MIN_SIZE_;
    nn[0]=l;
    result+=STD_string(nn)+"\n";
  }
  if(get_jdx_compatmode()==bruker) result+="<";
  result+=str;
  if(get_jdx_compatmode()==bruker) result+=">";
  ODINLOG(odinlog,normalDebug) << "returning >" << result << "<" << STD_endl;
  return result;
}


STD_string LDRserJDX::parse_string(const STD_string& parstring) const {
  Log<LDRcomp> odinlog("LDRserJDX","parse_string");
  STD_string result;
  ODINLOG(odinlog,normalDebug) << "str: >" << parstring << "<" << STD_endl;
  STD_string without_size_field;
  if(get_jdx_compatmode()==bruker) without_size_field=extract(parstring,"\n","");
  else without_size_field=parstring;
  ODINLOG(odinlog,normalDebug) << "without_size_field: >" << without_size_field << "<" << STD_endl;
  STD_string shrinked=shrink(without_size_field);
  ODINLOG(odinlog,normalDebug) << "shrinked: >" << shrinked << "<" << STD_endl;
  if (shrinked.length()>=2 && shrinked[(unsigned int)0]=='<' && shrinked[shrinked.length()-1]=='>') result=extract(without_size_field,"<",">",true);
  else result=without_size_field;
  return result;
}


STD_string LDRserJDX::get_blocklabel(const STD_string& parstring) const {
  STD_string result;
  STD_string parlabel=extract(parstring,"##","=");
  if(parlabel=="TITLE") {
    result=extract(parstring,"##TITLE=","\n");
  }
  return result;
}


STD_string LDRserJDX::get_blockbody(const STD_string& parstring, bool including_delimiters) const {
  STD_string body=extract(parstring,"##TITLE=","\n##END=",true); // Extract body of parameter block, regarding nested structure (blocks within blocks)
  if(including_delimiters) return "##TITLE="+body+"\n##END=";
  return body;
}


STD_string LDRserJDX::get_parlabel(const STD_string& parstring) const {
  STD_string parlabel(extract(parstring,"##","="));
//  if(parlabel=="END") return ""; // break while loop in calling code
  if (parlabel[0]=='$') {
    parlabel+="=";
    parlabel=extract(parlabel,"$","=");
  }
  if(parlabel=="TITLE") {
    parlabel=extract(parstring,"##TITLE=","\n");
  }
  return parlabel;
}

void LDRserJDX::remove_next_ldr(STD_string& parstring) const {

  if(noccur(parstring,"##")>1) {
    parstring=rmblock(parstring,"##","##",true,false,false,false);
  } else {
    parstring="";
  }

}


///////////////////////////////////////////////////////////////////////

STD_string LDRserXML::str_between_delimiters(const STD_string& parstring, STD_string& startdelim, STD_string& enddelim) const {
  Log<LDRcomp> odinlog("LDRserXML","str_between_delimiters");

  STD_string result;

  STD_string parlabel=get_parlabel(parstring);
  ODINLOG(odinlog,normalDebug) << "parlabel: >" << parlabel << "<" << STD_endl;

  STD_string argsstr=extract(parstring,"<"+parlabel,">");
  ODINLOG(odinlog,normalDebug) << "argsstr: >" << argsstr << "<" << STD_endl;

  startdelim="<"+parlabel+argsstr+">";
  enddelim="</"+parlabel+">";
  ODINLOG(odinlog,normalDebug) << "startdelim-enddelim: " << startdelim << "-" << enddelim << STD_endl;

  result=extract(parstring,startdelim,enddelim,true);

  return result;
}

STD_string LDRserXML::get_prefix(const LDRbase& ldr) const {
  STD_string result;
  result="<"+ldr.get_label()+">";
  if(ldr.get_typeInfo()=="LDRblock") result+="\n";
  return result;
}

STD_string LDRserXML::get_postfix(const LDRbase& ldr) const {
  STD_string result;
//  if(ldr.get_typeInfo()!="LDRblock") result+="\n";
  result+="</"+ldr.get_label()+">\n";
  return result;
}


STD_string LDRserXML::extract_valstring(const STD_string& parstring) const {
  Log<LDRcomp> odinlog("LDRserXML","extract_valstring");
  STD_string result;

  STD_string startdelim;
  STD_string enddelim;
  result=str_between_delimiters(parstring,startdelim,enddelim);

  ODINLOG(odinlog,normalDebug) << "result: >" << result << "<" << STD_endl;

  return result;
}


STD_string LDRserXML::remove_comments(const STD_string& parstring) const {
  STD_string result;

  result=rmblock(parstring, "<!--", "-->", true, true, true, true);
  result=rmblock(result, "<?", "?>", true, true, true, true);

  return result;
}

STD_string LDRserXML::escape_characters(const STD_string& parstring) const {
  STD_string result;
  result=replaceStr(parstring,"\"","&quot;");
  result=replaceStr(result,"<","&lt;");
  result=replaceStr(result,">","&gt;");
  result=replaceStr(result,"&","&amp;");
  return result;
}

STD_string LDRserXML::deescape_characters(const STD_string& parstring) const {
  STD_string result;
  result=replaceStr(parstring,"&quot;","\"");
  result=replaceStr(result,"&lt;","<");
  result=replaceStr(result,"&gt;",">");
  result=replaceStr(result,"&amp;","&");
  return result;
}


STD_string LDRserXML::get_blocklabel(const STD_string& parstring) const {
  Log<LDRcomp> odinlog("LDRserXML","get_blocklabel");
  STD_string result;

  STD_string blockbody=get_blockbody(parstring, false);
  ODINLOG(odinlog,normalDebug) << "blockbody: >" << blockbody << "<" << STD_endl;

  if(extract(blockbody,"<",">")!="") { // any tags inside?
    result=get_parlabel(parstring);
  }

  ODINLOG(odinlog,normalDebug) << "result: >" << result << "<" << STD_endl;

  return result;
}


STD_string LDRserXML::get_blockbody(const STD_string& parstring, bool including_delimiters) const {
  Log<LDRcomp> odinlog("LDRserXML","get_blockbody");
  STD_string result;

  STD_string startdelim;
  STD_string enddelim;
  result=str_between_delimiters(parstring,startdelim,enddelim);

  if(including_delimiters) result=startdelim+result+enddelim;

  return result;
}


STD_string LDRserXML::get_parlabel(const STD_string& parstring) const {
  Log<LDRcomp> odinlog("LDRserXML","get_parlabel");
  STD_string result;
  STD_string label_with_args=extract(parstring,"<",">",true);
  svector toks=tokens(label_with_args); // strip arguments
  if(toks.size()) result=toks[0];
  ODINLOG(odinlog,normalDebug) << "result: >" << result << "<" << STD_endl;
  return result;
}


void LDRserXML::remove_next_ldr(STD_string& parstring) const {
  Log<LDRcomp> odinlog("LDRserXML","remove_next_ldr");

  STD_string startdelim;
  STD_string enddelim;
  STD_string valstr=str_between_delimiters(parstring,startdelim,enddelim);

  ODINLOG(odinlog,normalDebug) << "startdelim-enddelim: " << startdelim << "-" << enddelim << STD_endl;

  parstring=rmblock(parstring,startdelim,enddelim,true,true,false,true);
}
